==> LostPixels-0.5-source.tar.gz: blinkensisters/lostpixels/game/software/tiles.cpp <==
// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer, Wolfgang Dautermann
//
// See License.txt for licensing information
//


#include "globals.h"
#include "tiles.h"
#include "levelhandler.h"
#include "blending.h"
#include "errorhandler.h"
#include "engine.h"

SDL_Surface *TILES_Surface;

void initTiles(char *fname) {
	char fullfname[MAX_FNAME_LENGTH];
	sprintf(fullfname, "%s", configGetPath(fname));
	SDL_Surface* temp = IMG_Load(fullfname);
	if(!temp) {
		DIE(ERROR_IMAGE_READ, fullfname);
	}
	TILES_Surface = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
}

void deInitTiles() {
	SDL_FreeSurface(TILES_Surface);
}

void paintSingleTile(Uint32 num, Sint32 x, Sint32 y) {
	if(num >= (Uint32)(TILES_Surface->h / TILESIZE)) {
		return; // Illegal tile
	}

	if(x < (1 - TILESIZE) || 
			y < (1 - TILESIZE) ||
			x > SCR_WIDTH || 
			y > SCR_HEIGHT) {
		return; // Tile is invisible so we ignore it
	}
	
	// Calculate offset in tiles-pixmap
	Uint32 xorigoffs = 0;
	Uint32 yorigoffs = 0;
	if(x < 0) {
		xorigoffs = -x;
	}
	if(y < 0) {
		yorigoffs = -y;
	}
	
	// Calculate HEIGHT/WIDTH of tiles to copy
	Uint32 width = TILESIZE;
	Uint32 height = TILESIZE;
	if(x > (SCR_WIDTH - TILESIZE)) {
		width = SCR_WIDTH - x;
	}
	if(y > (SCR_HEIGHT - TILESIZE)) {
		height = SCR_HEIGHT - y;
	}
	
	// -- Let's get blitting!!!
	if (SDL_MUSTLOCK(TILES_Surface))
		if (SDL_LockSurface(TILES_Surface) < 0)
			return;	

	Uint32 scrpitch = gScreen->pitch / 4;
	Uint32 tilpitch = TILES_Surface->pitch / 4;
	
	/*
	for(Uint32 ybg = yorigoffs; ybg < height; ybg++) { 
		for(Uint32 xbg = xorigoffs; xbg < width; xbg++) {
			((Uint32*)gScreen->pixels)[xbg + x + (y + ybg) * scrpitch] = 
				((Uint32*)TILES_Surface->pixels)[xbg + (ybg + num*TILESIZE) * tilpitch];
		}
	}
	*/
	
	Uint32 color;
	for(Uint32 ybg = yorigoffs; ybg < height; ybg++) { 
		for(Uint32 xbg = xorigoffs; xbg < width; xbg++) {
			color = ((Uint32*)TILES_Surface->pixels)[xbg + (ybg + num*TILESIZE) * tilpitch];
			
			if(color != 0x0000ff00) {
				((Uint32*)gScreen->pixels)[xbg + x + (y + ybg) * scrpitch] = color;
			}
			
		}
	}
	
	// Unlock if needed
	if (SDL_MUSTLOCK(TILES_Surface))
		SDL_UnlockSurface(TILES_Surface);
	
		
	return;
}

Uint32 getNumOfTiles() {
	return (TILES_Surface->h / TILESIZE + 1);
}


void paintLevelTiles(Uint32 xoffs, Uint32 yoffs) {
	DISPLAYLIST *tthis = lhandle.dlist;
	Sint32 xpos;
	Sint32 ypos;
	while(tthis) {
		xpos = tthis->x - xoffs;
		ypos = tthis->y - yoffs;		
		if(xpos > -TILESIZE && xpos < SCR_WIDTH && ypos > -TILESIZE && ypos < SCR_HEIGHT) {
			paintSingleTile(tthis->type, xpos, ypos);
		}
		tthis = tthis->next;
	}
}

void paintSpecialTiles(Uint32 xoffs, Uint32 yoffs, bool canExit) {
	Sint32 xpos;
	Sint32 ypos;
	Uint32 pitch = gScreen->pitch / 4;
	static Sint32 startcoloffs = 0;
	static Sint32 goalcoloffs = 0;
	Uint32 color;
	Uint32 tilecolor;
	static Sint32 lasttick = 0;
	Sint32 tick = SDL_GetTicks();
	
	// Ignore delays over 1 second
	if((lasttick + 1000) < tick) {
		lasttick = tick;
	}
	
	while(lasttick < tick) {
		startcoloffs = (startcoloffs + 1) % TILESIZE;
		goalcoloffs--;
		if(goalcoloffs < 0) {
			goalcoloffs = TILESIZE - 1;
		}
		lasttick += SPECIALTILEDELAY;
	}
	
	
	if(!canPaintSpecialTiles) {
		return;
	}
	
/*	
	// START-TILE
	xpos = lhandle.start_x - xoffs;
	ypos = lhandle.start_y - yoffs;
	if(xpos > -TILESIZE && xpos < SCR_WIDTH && ypos > -TILESIZE && ypos < SCR_HEIGHT) {
		for(Sint32 x = xpos; x < (xpos + TILESIZE); x++) {
			if(x > 0 && x < SCR_WIDTH) {
				for(Sint32 y = ypos; y < (ypos + TILESIZE); y++) {
					if(y > 0 && y < SCR_HEIGHT) {
						color =  blend_mul(((Uint32*)gScreen->pixels)[x + y * pitch], 0x00707040);
						tilecolor = (((TILESIZE - (abs(x - xpos - TILESIZE/2) + abs(y - ypos - TILESIZE/2))) + startcoloffs) % TILESIZE) * 16;
						((Uint32*)gScreen->pixels)[x + y * pitch] =  blend_add(color, tilecolor);
					}
				}
			}
		}
	}
*/	
	
	// GOAL-TILE
	xpos = lhandle.goal_x - xoffs;
	ypos = lhandle.goal_y - yoffs;
	if(xpos > -TILESIZE && xpos < SCR_WIDTH && ypos > -TILESIZE && ypos < SCR_HEIGHT) {
		for(Sint32 x = xpos; x < (xpos + TILESIZE); x++) {
			if(x > 0 && x < SCR_WIDTH) {
				for(Sint32 y = ypos; y < (ypos + TILESIZE); y++) {
					if(y > 0 && y < SCR_HEIGHT) {
						if(canExit) {
							tilecolor = ((((TILESIZE - (abs(x - xpos - TILESIZE/2) + abs(y - ypos - TILESIZE/2))) + goalcoloffs) % TILESIZE) * 16) << 8;
							color =  blend_mul(((Uint32*)gScreen->pixels)[x + y * pitch], 0x00704070);
						} else {
							tilecolor = ((((TILESIZE - (abs(x - xpos - TILESIZE/2) + abs(y - ypos - TILESIZE/2))) + goalcoloffs) % TILESIZE) * 16) << 16;
							color =  blend_mul(((Uint32*)gScreen->pixels)[x + y * pitch], 0x00407070);
						}
						((Uint32*)gScreen->pixels)[x + y * pitch] =  blend_add(color, tilecolor);
					}
				}
			}
		}
	}
}

==> LostPixels-0.5.2-source.tar.gz: blinkensisters/lostpixels/game/software/tiles.cpp <==
// BlinkenSisters - Hunt for the Lost Pixels
//     Bringing back the fun of the 80s
//
// (C) 2005-07 Rene Schickbauer, Wolfgang Dautermann
//
// See License.txt for licensing information
//


#include "globals.h"
#include "tiles.h"
#include "levelhandler.h"
#include "blending.h"
#include "errorhandler.h"
#include "engine.h"
#include "convert.h"

SDL_Surface *TILES_Surface;

void initTiles(const char *fname) {
	char fullfname[MAX_FNAME_LENGTH];
	sprintf(fullfname, "%s", configGetPath(fname));
	SDL_Surface* temp = IMG_Load(fullfname);
	if(!temp) {
		DIE(ERROR_IMAGE_READ, fullfname);
	}
	TILES_Surface = convertToBSSurface(temp);
	SDL_FreeSurface(temp);
}

void deInitTiles() {
	SDL_FreeSurface(TILES_Surface);
}

void paintSingleTile(const Uint32 num, const Sint32 x, const Sint32 y) {
	if(num >= (Uint32)(TILES_Surface->h / TILESIZE)) {
		return; // Illegal tile
	}

	if(x < (1 - TILESIZE) || 
			y < (1 - TILESIZE) ||
			x > SCR_WIDTH || 
			y > SCR_HEIGHT) {
		return; // Tile is invisible so we ignore it
	}
	
    static SDL_Rect src;
	static SDL_Rect dest;

    src.x = 0;
    src.y = TILESIZE * num;
    src.w = dest.w = src.h = dest.h = TILESIZE;	
    dest.x = x;
    dest.y = y;
    
	SDL_BlitSurface(TILES_Surface, &src, gScreen, &dest);

	return;
}

Uint32 getNumOfTiles() {
	return (TILES_Surface->h / TILESIZE + 1);
}


void paintLevelTiles(const Uint32 xoffs, const Uint32 yoffs) {
	DISPLAYLIST *tthis = lhandle.dlist;
	Sint32 xpos;
	Sint32 ypos;

	if (SDL_MUSTLOCK(TILES_Surface))
		if (SDL_LockSurface(TILES_Surface) < 0)
			return;	
	
	while(tthis) {
		xpos = tthis->x - xoffs;
		ypos = tthis->y - yoffs;		
		if(tthis->type < 10 && xpos > -TILESIZE && xpos < SCR_WIDTH && ypos > -TILESIZE && ypos < SCR_HEIGHT) {
			paintSingleTile(tthis->type, xpos, ypos);
		}
		tthis = tthis->next;
	}
	
	// Unlock if needed
	if (SDL_MUSTLOCK(TILES_Surface))
		SDL_UnlockSurface(TILES_Surface);
	
	
}
